==> dom_builder-2.0.7.tar.gz: test/dom_builder_domtest.dart <==
import 'package:dom_builder/dom_builder.dart';

class TestNodeGenerator extends ElementGenerator<TestElem> {
  @override
  final String tag;
  final String classes;

  TestNodeGenerator(this.tag, this.classes);

  @override
  TestElem generate(
      DOMGenerator<dynamic> domGenerator,
      DOMTreeMap<dynamic> treeMap,
      String? tag,
      DOMElement? domParent,
      parent,
      DOMNode domNode,
      Map<String, DOMAttribute> attributes,
      TestElem? contentHolder,
      List<DOMNode>? contentNodes,
      DOMContext<dynamic>? context) {
    var attributesAsString =
        attributes.map((key, value) => MapEntry(key, value.value ?? ''));

    var elem = TestElem(tag)..attributes.addAll(attributesAsString);

    var prevClass = elem.attributes['class'] ?? '';

    elem.attributes['class'] = (classes + ' ' + prevClass).trim();

    if (contentHolder != null) {
      elem.add(TestText(contentHolder.text));
    }

    return elem;
  }

  @override
  DOMElement revert(
      DOMGenerator<dynamic> domGenerator,
      DOMTreeMap<dynamic>? treeMap,
      DOMElement? domParent,
      TestElem? parent,
      TestElem? node) {
    var tag = node!.tag;
    var prevClass = node.attributes['class'];

    if (prevClass != null && prevClass.contains(tag!)) {
      prevClass = prevClass.replaceFirst(tag, '').trim();
      node.attributes['class'] = prevClass;
    }

    return DOMElement(tag, attributes: node.attributes);
  }
}

abstract class TestNode {
  static int instanceIDCount = 0;

  final int instanceID = ++instanceIDCount;

  TestNode? parent;

  String get text;

  TestNode copy();

  String outerHTML();
}

class TestText extends TestNode {
  @override
  String text;

  TestText(String? text) : text = text ?? '';

  TestElem get asTestElem => TestElem('span')..add(TestText(text));

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is TestText &&
          runtimeType == other.runtimeType &&
          text == other.text;

  @override
  int get hashCode => instanceID;

  @override
  String toString() {
    return text;
  }

  @override
  TestText copy() => TestText(text);

  @override
  String outerHTML() => text;
}

class TestElem extends TestNode {
  final String? tag;

  TestElem(this.tag);

  @override
  TestElem copy() {
    var copy = TestElem(tag);

    nodes.map((e) => e.copy()).forEach((e) {
      copy.add(e);
    });

    return copy;
  }

  final List<TestNode> _nodes = [];

  List<TestNode> get nodes => List.unmodifiable(_nodes);

  int get nodesLength => _nodes.length;

  @override
  String get text {
    if (_nodes.isEmpty) return '';
    return _nodes.map((e) => e.text).join('');
  }

  TestNode get(int index) => _nodes[index];

  void addAll(Iterable<TestNode> nodes) {
    _nodes.addAll(nodes);
  }

  bool contains(TestNode? node) => _nodes.contains(node);

  void add(TestNode node) {
    _nodes.add(node);
    _setParent(node, this);
  }

  bool remove(TestNode? node) {
    if (_nodes.remove(node)) {
      _setParentNull(node!);
      return true;
    }
    return false;
  }

  void _setParent(TestNode node, TestElem parent) {
    node.parent = parent;
  }

  void _setParentNull(TestNode node) {
    node.parent = null;
  }

  TestNode removeAt(int index) {
    var node = _nodes.removeAt(index);
    _setParentNull(node);
    return node;
  }

  void insertAt(int index, TestNode node) {
    _nodes.insert(index, node);
    _setParent(node, this);
  }

  int indexOf(TestNode node) {
    return _nodes.indexOf(node);
  }

  void clear() {
    _nodes.forEach(_setParentNull);
    _nodes.clear();
  }

  final Map<String, String> attributes = {};

  String get asHTML => outerHTML();

  @override
  String outerHTML() {
    var attrs =
        attributes.entries.map((e) => '${e.key}="${e.value}"').join(' ');
    if (attrs.isNotEmpty) attrs = ' $attrs';

    var html = '<$tag$attrs>';

    for (var node in _nodes) {
      html += node.outerHTML();
    }

    html += '</$tag>';

    return html;
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is TestElem &&
          runtimeType == other.runtimeType &&
          tag == other.tag &&
          text == other.text;

  @override
  int get hashCode => tag.hashCode ^ instanceID;

  @override
  String toString() {
    return 'TestElem{tag: $tag, nodes: $nodes, text: $text, attributes: $attributes}';
  }
}

class TestGenerator extends DOMGenerator<TestNode> {
  @override
  TestNode? getNodeParent(TestNode? node) {
    return node!.parent;
  }

  @override
  List<TestNode> getElementNodes(TestNode? element) {
    if (element is TestElem) {
      return List.from(element.nodes);
    }
    return [];
  }

  @override
  String? getElementTag(TestNode? element) {
    if (element is TestElem) {
      return element.tag;
    }
    return null;
  }

  @override
  String? getElementValue(TestNode? element) {
    if (element == null) return null;
    return element.text;
  }

  @override
  String getElementOuterHTML(TestNode? element) {
    return element!.outerHTML();
  }

  @override
  Map<String, String>? getElementAttributes(TestNode? element) {
    if (element is TestElem) {
      return Map.fromEntries(element.attributes.entries);
    }
    return null;
  }

  @override
  bool addChildToElement(TestNode? parent, TestNode? child) {
    if (parent is TestElem && !parent.contains(child)) {
      parent.add(child!);
      return true;
    }
    return false;
  }

  @override
  bool removeChildFromElement(TestNode parent, TestNode? child) {
    if (parent is TestElem) {
      parent.remove(child);
      return true;
    }
    return false;
  }

  @override
  bool replaceChildElement(
      TestNode parent, TestNode? child1, List<TestNode>? child2) {
    if (parent is TestElem) {
      var idx = parent.indexOf(child1!);
      if (idx >= 0) {
        parent.removeAt(idx);
        for (var i = 0; i < child2!.length; ++i) {
          var e = child2[i];
          parent.insertAt(idx + i, e);
        }
      }
      return true;
    }
    return false;
  }

  @override
  bool canHandleExternalElement(externalElement) {
    return externalElement is TestElem || externalElement is TestText;
  }

  @override
  List<TestNode>? addExternalElementToElement(
      TestNode element, Object? externalElement) {
    if (element is TestElem) {
      if (externalElement is TestElem) {
        element.add(externalElement);
        return [externalElement];
      } else if (externalElement is TestText) {
        var testElem = externalElement.asTestElem;
        element.add(testElem);
        return [testElem];
      }
    }
    return null;
  }

  @override
  TestNode? appendElementText(TestNode element, String? text) {
    if (element is TestElem) {
      var textElem = TestText(text);
      element.add(textElem);
      return textElem;
    }
    return null;
  }

  @override
  TestNode createTextNode(String? text) {
    return TestText(text);
  }

  @override
  bool isTextNode(TestNode? node) => node is TestText;

  @override
  bool containsNode(TestNode parent, TestNode? node) {
    if (node == null) return false;

    if (parent is TestElem) {
      return parent.nodes.contains(node);
    }

    return false;
  }

  @override
  TestElem createElement(String? tag, [DOMElement? domElement]) {
    return TestElem(tag);
  }

  @override
  String getNodeText(TestNode? node) {
    return node!.text;
  }

  @override
  void setAttribute(TestNode element, String attrName, String? attrVal) {
    if (element is TestElem) {
      element.attributes[attrName] = attrVal!;
    }
  }

  @override
  String? getAttribute(TestNode element, String attrName) {
    if (element is TestElem) {
      return element.attributes[attrName];
    }
    return null;
  }

  @override
  String buildElementHTML(TestNode element) {
    if (element is TestElem) {
      return element.asHTML;
    }
    return '';
  }

  @override
  DOMNodeRuntime<TestNode> createDOMNodeRuntime(
      DOMTreeMap<TestNode> treeMap, DOMNode? domNode, TestNode node) {
    return TestNodeRuntime(treeMap, domNode, node as TestElem);
  }
}

class TestNodeRuntime extends DOMNodeRuntime<TestNode> {
  TestNodeRuntime(DOMTreeMap<TestNode> treeMap, DOMNode? domNode, TestElem node)
      : super(treeMap, domNode, node);

  @override
  String? get tagName {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.tag;
    }
    return null;
  }

  @override
  String get text {
    return node!.text;
  }

  @override
  set text(String? value) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.add(TestText(value));
    } else if (node is TestText) {
      var textElem = node as TestText;
      textElem.text = value ?? '';
    }
  }

  @override
  String get value => text;

  @override
  set value(String? value) {
    text = value;
  }

  @override
  String? getAttribute(String name) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.attributes[name];
    }
    return null;
  }

  @override
  void setAttribute(String name, String value) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.attributes[name] = value;
    }
  }

  @override
  void removeAttribute(String name) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.attributes.remove(name);
    }
  }

  @override
  List<TestNode> get children {
    if (node is TestElem) {
      var element = node as TestElem;
      return List.from(element.nodes);
    }
    return [];
  }

  @override
  int get nodesLength {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.nodes.length;
    }
    return 0;
  }

  @override
  TestNode? getNodeAt(int index) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.nodes[index];
    }
    return null;
  }

  @override
  void add(TestNode? child) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.add(child!);
    }
  }

  @override
  void clear() {
    if (node is TestElem) {
      var element = node as TestElem;
      element.clear();
    }
  }

  @override
  int indexOf(TestNode? child) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.indexOf(child!);
    }
    return -1;
  }

  @override
  void insertAt(int index, TestNode? child) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.insertAt(index, child!);
    }
  }

  @override
  bool removeNode(TestNode? child) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.remove(child);
    }
    return false;
  }

  @override
  TestElem? removeAt(int index) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.removeAt(index) as TestElem?;
    }
    return null;
  }

  @override
  void addClass(String? className) {}

  @override
  List<String> get classes => [];

  @override
  void clearClasses() {}

  @override
  bool removeClass(String? className) => false;

  @override
  TestNode copy() {
    return node!.copy();
  }

  @override
  bool absorbNode(TestNode? other) {
    if (node is TestText) {
      if (other is TestText) {
        var textNode = node as TestText;
        textNode.text += other.text;
        other.text = '';
        return true;
      }
    } else if (node is TestElem) {
      var elemNode = node as TestElem?;

      if (other is TestElem) {
        elemNode!.addAll(other.nodes);
        other.clear();
        return true;
      }
    }

    throw UnimplementedError('$node > $other');
  }

  @override
  int get indexInParent {
    if (hasParent) {
      return parentRuntime!.indexOf(node!);
    }
    return -1;
  }

  @override
  bool get isStringElement {
    if (node is TestText) {
      return true;
    } else if (node is TestElem) {
      var elem = node as TestElem;
      return DOMElement.isStringTagName(elem.tag);
    }
    return false;
  }
}

class TestActionExecutor extends DOMActionExecutor<TestNode> {
  @override
  TestNode call(String name, List<String> parameters, TestNode? target,
      TestNode? self, DOMTreeMap? treeMap, DOMContext? context) {
    throw UnimplementedError();
  }

  @override
  TestNode selectByID(String id, TestNode? target, TestNode? self,
      DOMTreeMap? treeMap, DOMContext? context) {
    throw UnimplementedError();
  }
}

==> dom_builder-2.1.3.tar.gz: test/dom_builder_domtest.dart <==
import 'package:dom_builder/dom_builder.dart';

class TestNodeGenerator extends ElementGenerator<TestElem> {
  @override
  final String tag;
  final String classes;

  TestNodeGenerator(this.tag, this.classes);

  @override
  TestElem generate(
      DOMGenerator<dynamic> domGenerator,
      DOMTreeMap<dynamic> treeMap,
      String? tag,
      DOMElement? domParent,
      parent,
      DOMNode domNode,
      Map<String, DOMAttribute> attributes,
      TestElem? contentHolder,
      List<DOMNode>? contentNodes,
      DOMContext<dynamic>? context) {
    var attributesAsString =
        attributes.map((key, value) => MapEntry(key, value.value ?? ''));

    var elem = TestElem(tag)..attributes.addAll(attributesAsString);

    var prevClass = elem.attributes['class'] ?? '';

    elem.attributes['class'] = '$classes $prevClass'.trim();

    if (contentHolder != null) {
      elem.add(TestText(contentHolder.text));
    }

    return elem;
  }

  @override
  DOMElement revert(
      DOMGenerator<dynamic> domGenerator,
      DOMTreeMap<dynamic>? treeMap,
      DOMElement? domParent,
      TestElem? parent,
      TestElem? node) {
    var tag = node!.tag;
    var prevClass = node.attributes['class'];

    if (prevClass != null && prevClass.contains(tag!)) {
      prevClass = prevClass.replaceFirst(tag, '').trim();
      node.attributes['class'] = prevClass;
    }

    return DOMElement(tag, attributes: node.attributes);
  }
}

abstract class TestNode {
  static int instanceIDCount = 0;

  final int instanceID = ++instanceIDCount;

  TestNode? parent;

  String get text;

  TestNode copy();

  String outerHTML();
}

class TestText extends TestNode {
  @override
  String text;

  TestText(String? text) : text = text ?? '';

  TestElem get asTestElem => TestElem('span')..add(TestText(text));

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is TestText &&
          runtimeType == other.runtimeType &&
          text == other.text;

  @override
  int get hashCode => instanceID;

  @override
  String toString() {
    return text;
  }

  @override
  TestText copy() => TestText(text);

  @override
  String outerHTML() => text;
}

class TestElem extends TestNode {
  final String? tag;

  TestElem(this.tag);

  @override
  TestElem copy() {
    var copy = TestElem(tag);

    nodes.map((e) => e.copy()).forEach((e) {
      copy.add(e);
    });

    return copy;
  }

  final List<TestNode> _nodes = [];

  List<TestNode> get nodes => List.unmodifiable(_nodes);

  int get nodesLength => _nodes.length;

  @override
  String get text {
    if (_nodes.isEmpty) return '';
    return _nodes.map((e) => e.text).join('');
  }

  TestNode get(int index) => _nodes[index];

  void addAll(Iterable<TestNode> nodes) {
    _nodes.addAll(nodes);
  }

  bool contains(TestNode? node) => _nodes.contains(node);

  void add(TestNode node) {
    _nodes.add(node);
    _setParent(node, this);
  }

  bool remove(TestNode? node) {
    if (_nodes.remove(node)) {
      _setParentNull(node!);
      return true;
    }
    return false;
  }

  void _setParent(TestNode node, TestElem parent) {
    node.parent = parent;
  }

  void _setParentNull(TestNode node) {
    node.parent = null;
  }

  TestNode removeAt(int index) {
    var node = _nodes.removeAt(index);
    _setParentNull(node);
    return node;
  }

  void insertAt(int index, TestNode node) {
    _nodes.insert(index, node);
    _setParent(node, this);
  }

  int indexOf(TestNode node) {
    return _nodes.indexOf(node);
  }

  void clear() {
    _nodes.forEach(_setParentNull);
    _nodes.clear();
  }

  final Map<String, String> attributes = {};

  String get asHTML => outerHTML();

  @override
  String outerHTML() {
    var attrs =
        attributes.entries.map((e) => '${e.key}="${e.value}"').join(' ');
    if (attrs.isNotEmpty) attrs = ' $attrs';

    var html = '<$tag$attrs>';

    for (var node in _nodes) {
      html += node.outerHTML();
    }

    html += '</$tag>';

    return html;
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is TestElem &&
          runtimeType == other.runtimeType &&
          tag == other.tag &&
          text == other.text;

  @override
  int get hashCode => tag.hashCode ^ instanceID;

  @override
  String toString() {
    return 'TestElem{tag: $tag, nodes: $nodes, text: $text, attributes: $attributes}';
  }
}

class TestGenerator extends DOMGenerator<TestNode> {
  @override
  TestNode? getNodeParent(TestNode? node) {
    return node!.parent;
  }

  @override
  List<TestNode> getElementNodes(TestNode? element) {
    if (element is TestElem) {
      return List.from(element.nodes);
    }
    return [];
  }

  @override
  String? getElementTag(TestNode? element) {
    if (element is TestElem) {
      return element.tag;
    }
    return null;
  }

  @override
  String? getElementValue(TestNode? element) {
    if (element == null) return null;
    return element.text;
  }

  @override
  String getElementOuterHTML(TestNode? element) {
    return element!.outerHTML();
  }

  @override
  Map<String, String>? getElementAttributes(TestNode? element) {
    if (element is TestElem) {
      return Map.fromEntries(element.attributes.entries);
    }
    return null;
  }

  @override
  bool addChildToElement(TestNode? parent, TestNode? child) {
    if (parent is TestElem && !parent.contains(child)) {
      parent.add(child!);
      return true;
    }
    return false;
  }

  @override
  bool removeChildFromElement(TestNode parent, TestNode? child) {
    if (parent is TestElem) {
      parent.remove(child);
      return true;
    }
    return false;
  }

  @override
  bool replaceChildElement(
      TestNode parent, TestNode? child1, List<TestNode>? child2) {
    if (parent is TestElem) {
      var idx = parent.indexOf(child1!);
      if (idx >= 0) {
        parent.removeAt(idx);
        for (var i = 0; i < child2!.length; ++i) {
          var e = child2[i];
          parent.insertAt(idx + i, e);
        }
      }
      return true;
    }
    return false;
  }

  @override
  bool canHandleExternalElement(externalElement) {
    return externalElement is TestElem || externalElement is TestText;
  }

  @override
  List<TestNode>? addExternalElementToElement(
      TestNode element, Object? externalElement) {
    if (element is TestElem) {
      if (externalElement is TestElem) {
        element.add(externalElement);
        return [externalElement];
      } else if (externalElement is TestText) {
        var testElem = externalElement.asTestElem;
        element.add(testElem);
        return [testElem];
      }
    }
    return null;
  }

  @override
  TestNode? appendElementText(TestNode element, String? text) {
    if (element is TestElem) {
      var textElem = TestText(text);
      element.add(textElem);
      return textElem;
    }
    return null;
  }

  @override
  TestNode createTextNode(String? text) {
    return TestText(text);
  }

  @override
  bool isTextNode(TestNode? node) => node is TestText;

  @override
  bool containsNode(TestNode parent, TestNode? node) {
    if (node == null) return false;

    if (parent is TestElem) {
      return parent.nodes.contains(node);
    }

    return false;
  }

  @override
  TestElem createElement(String? tag, [DOMElement? domElement]) {
    return TestElem(tag);
  }

  @override
  String getNodeText(TestNode? node) {
    return node!.text;
  }

  @override
  void setAttribute(TestNode element, String attrName, String? attrVal) {
    if (element is TestElem) {
      element.attributes[attrName] = attrVal!;
    }
  }

  @override
  String? getAttribute(TestNode element, String attrName) {
    if (element is TestElem) {
      return element.attributes[attrName];
    }
    return null;
  }

  @override
  String buildElementHTML(TestNode element) {
    if (element is TestElem) {
      return element.asHTML;
    }
    return '';
  }

  @override
  DOMNodeRuntime<TestNode> createDOMNodeRuntime(
      DOMTreeMap<TestNode> treeMap, DOMNode? domNode, TestNode node) {
    return TestNodeRuntime(treeMap, domNode, node as TestElem);
  }
}

class TestNodeRuntime extends DOMNodeRuntime<TestNode> {
  TestNodeRuntime(DOMTreeMap<TestNode> treeMap, DOMNode? domNode, TestElem node)
      : super(treeMap, domNode, node);

  @override
  String? get tagName {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.tag;
    }
    return null;
  }

  @override
  String get text {
    return node!.text;
  }

  @override
  set text(String? value) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.add(TestText(value));
    } else if (node is TestText) {
      var textElem = node as TestText;
      textElem.text = value ?? '';
    }
  }

  @override
  String get value => text;

  @override
  set value(String? value) {
    text = value;
  }

  @override
  String? getAttribute(String name) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.attributes[name];
    }
    return null;
  }

  @override
  void setAttribute(String name, String value) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.attributes[name] = value;
    }
  }

  @override
  void removeAttribute(String name) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.attributes.remove(name);
    }
  }

  @override
  List<TestNode> get children {
    if (node is TestElem) {
      var element = node as TestElem;
      return List.from(element.nodes);
    }
    return [];
  }

  @override
  int get nodesLength {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.nodes.length;
    }
    return 0;
  }

  @override
  TestNode? getNodeAt(int index) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.nodes[index];
    }
    return null;
  }

  @override
  void add(TestNode? child) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.add(child!);
    }
  }

  @override
  void clear() {
    if (node is TestElem) {
      var element = node as TestElem;
      element.clear();
    }
  }

  @override
  int indexOf(TestNode? child) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.indexOf(child!);
    }
    return -1;
  }

  @override
  void insertAt(int index, TestNode? child) {
    if (node is TestElem) {
      var element = node as TestElem;
      element.insertAt(index, child!);
    }
  }

  @override
  bool removeNode(TestNode? child) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.remove(child);
    }
    return false;
  }

  @override
  TestElem? removeAt(int index) {
    if (node is TestElem) {
      var element = node as TestElem;
      return element.removeAt(index) as TestElem?;
    }
    return null;
  }

  @override
  void addClass(String? className) {}

  @override
  List<String> get classes => [];

  @override
  void clearClasses() {}

  @override
  bool removeClass(String? className) => false;

  @override
  TestNode copy() {
    return node!.copy();
  }

  @override
  bool absorbNode(TestNode? other) {
    if (node is TestText) {
      if (other is TestText) {
        var textNode = node as TestText;
        textNode.text += other.text;
        other.text = '';
        return true;
      }
    } else if (node is TestElem) {
      var elemNode = node as TestElem?;

      if (other is TestElem) {
        elemNode!.addAll(other.nodes);
        other.clear();
        return true;
      }
    }

    throw UnimplementedError('$node > $other');
  }

  @override
  int get indexInParent {
    if (hasParent) {
      return parentRuntime!.indexOf(node!);
    }
    return -1;
  }

  @override
  bool get isStringElement {
    if (node is TestText) {
      return true;
    } else if (node is TestElem) {
      var elem = node as TestElem;
      return DOMElement.isStringTagName(elem.tag);
    }
    return false;
  }
}

class TestActionExecutor extends DOMActionExecutor<TestNode> {
  @override
  TestNode call(String name, List<String> parameters, TestNode? target,
      TestNode? self, DOMTreeMap? treeMap, DOMContext? context) {
    throw UnimplementedError();
  }

  @override
  TestNode selectByID(String id, TestNode? target, TestNode? self,
      DOMTreeMap? treeMap, DOMContext? context) {
    throw UnimplementedError();
  }
}
